# License Module Injection Start
module DM::Bevel
LIC_FILE = 'bevel.lic'.freeze
LIC_DIR = 'Bevel'.freeze


# This method is generated by 'build_license.rb'
def self.load_c_extensions
  dir = File.dirname(__FILE__)
  su_version = Sketchup.version.to_i
  ruby_ver = case su_version
               when 24
                 '3.2'
               when 21, 22, 23
                 '2.7'
               when 19, 20
                 '2.5'
               when 18, 17
                  '2.2'
               when 16
                 '2.0'
               else
                 '3.2'
               end

  if Object::RUBY_PLATFORM =~ /darwin/i
    live_so_path = File.join(dir, 'so', 'bundle', ruby_ver, 'MSS_Licensing.bundle')
    source_so_path = File.join(dir, 'so', 'bundle', ruby_ver, 'MSS_Licensing_source.bundle')
    if !File.exist?(live_so_path) || !FileUtils.compare_file(source_so_path, live_so_path)
      File.unlink(live_so_path) if File.exist?(live_so_path)
      FileUtils.cp(source_so_path, live_so_path)
    end
  else
    arch = Sketchup.respond_to?(:is_64bit?) && Sketchup.is_64bit? ? '64bits' : '32bits'
    so_path = File.join(dir, 'so', ruby_ver, arch)
    source_so_path = File.join(so_path, 'MSS_Licensing_source.so')
    live_so_path = File.join(so_path, 'MSS_Licensing.so')
    #copy source c++ code to live (this is needed so users can update the plugin)
    #If we only have the source file it cannot be overwritten while SketchUp is in use
    if !File.exist?(live_so_path) || !FileUtils.compare_file(source_so_path, live_so_path)
      File.unlink(live_so_path) if File.exist?(live_so_path)
      FileUtils.cp(source_so_path, live_so_path)
    end
  end
  valid_checksums = {
    'DM_Bevel/so/2.2/64bits/MSS_Licensing_source.so' => '05a1da1d7f9d914a9b9c62cece5b4d2136751431f8d733bd8b27883a0b04215f',
'DM_Bevel/so/2.5/64bits/MSS_Licensing_source.so' => 'f77d57a3e8871e2762b435579c8928241128f3eca7b12cc2f8cbf2d2e85ec68d',
'DM_Bevel/so/2.7/64bits/MSS_Licensing_source.so' => '0553d53dc0694eb03dc300bf060f07d5b85616f1782f90f24b49e86ea1cce381',
'DM_Bevel/so/3.2/64bits/MSS_Licensing_source.so' => 'bfcc2fddc58f726cf2227ab77a9a69bbcb9655b1323ad191920d3d79f03aaf20',
'DM_Bevel/so/bundle/2.2/MSS_Licensing_source.bundle' => '5b8a77c4afc81ab19d58b85c2f73bcf1da0cdd662430a8d39be001048b66c3f9',
'DM_Bevel/so/bundle/2.5/MSS_Licensing_source.bundle' => '7c68a44a645b20f8e6e2f89710481c6919731a719a7327f1bc4ea553a2a142fb',
'DM_Bevel/so/bundle/2.7/MSS_Licensing_source.bundle' => '32a6649385aa01a1a11c2e3e3d9c5ed1e71c41bb5d9dc0ebf65823529301b1f6',
'DM_Bevel/so/bundle/3.2/MSS_Licensing_source.bundle' => '79b77ea5e70af39eabb26abdcb6be76d018fe1f893aeef3c47e9f7f6b5cc0b58'
  }
  cs = nil
  valid_checksums.each { |key, val|
    if source_so_path.downcase.include?(key.downcase)
      cs = Digest::SHA256.file(source_so_path).hexdigest
      raise 'ChecksumError' unless cs == cs
    end
  }
  raise 'ChecksumError' unless cs # Make sure matching checksum was calculated
  require live_so_path #load c++ code
  @did_load_extensions = true # Flag that the plugin can check for to determine whether to create the UI
  rescue Errno::EACCES => e
    UI.messagebox "Failed to load #{PLUGIN_NAME}. Please ensure the most recent version is installed, close all SketchUp windows and restart to finish the update."
  rescue Exception => e
    if e.message == 'ChecksumError'
      # Should only occur if plugin was hacked so throw an obscure message with no console output
      UI.messagebox "Failed to load #{PLUGIN_NAME}. Please close all SketchUp windows and restart."
      @did_load_extensions = false
    else
     raise
    end
end

require 'time'
require 'digest'
require 'fileutils'
require 'logger'
require 'stringio'
require 'tmpdir'
require 'net/http'
require 'uri'
require 'open-uri'
require 'json'

# This exception is raised and caught by the PLUGIN when the c++ license check fails
class InvalidLicenseException < RuntimeError; end

# RUBY 2.5 removed this function: https://github.com/rails/rails/pull/31462/files
# https://medium.com/@maciejmensfeld/ruby-2-5-0-upgrade-remarks-100d7c0dd73d
# Generates log files like: name-YYYYMMDD-9004-bxfuvf.log
def self.tmpname(name)
  t = Time.now.strftime("%Y%m%d")
  "#{name}-#{t}-#{$$}-#{rand(0x100000000).to_s(36)}"
end

# This will define a standard 'check_license' method that is exposed to Ruby
# Note that this does an offline check online and should only be used in addition
# to the other checks implemented inside c++
# Call this from a plugin by using 'PLUGIN.check_license'.  Use this check to prevent
# a dialog from opening or a tool from starting
def self.check_license
  lic = Licensing::License.new
  state = lic.get_state
  case state
  when Licensing::LICENSED, Licensing::TRIAL
    return true
  when Licensing::TRIAL_EXPIRED
    Licensing.on_trial_expired
    return false
  when Licensing::LICENSE_EXPIRED
    Licensing.on_license_expired
    return false
  else
    Licensing.on_invalid_license_found
    return false
  end
end

module Http

  # Defines a simple method that can be used to check if user is online along
  # with success and error code blocks
  def self.ping(on_success, on_error)
    if Sketchup.version.to_i < 19
      # Ping our server to test if user is online.  Note that on SU2018
      # the Sketchup.is_online method no longer works and for Mac, SU2017 and SU2018,
      # calling Sketchup.is_online inside an htmldialog callback causes SketchUp
      # to permanently hang, hence the logic below
      @ping_req = Request.new("https://0.0.0.0/")
      @ping_req.start do |req, resp|
        if resp.status_code != 0 # online
          on_success.call
        else
          # no connection
          on_error.call
        end
      end
    else
      if Sketchup.is_online
        on_success.call
      else
        on_error.call
      end
    end
  end

  class Request

    attr_accessor :body, :method


    def initialize(url, method = 'GET')
      @url = url
      @method = method
      uri = URI.parse(url)
      @host = uri.host
      @path = uri.path
      @query = uri.query
      @port = uri.port
      @scheme = uri.scheme
      @body = ""
      @req = create_request
    end

    def create_request
      Sketchup::Http::Request.new(@url, @method)
    end

    def set_download_progress_callback(&block)
      @req.set_download_progress_callback(&block)
    end

    def body=(value)
      @body = value
    end

    def cancel
      @req.cancel
    end

    def headers
      @req.headers
    end

    def headers=(hash)
      @req.headers = hash
    end

    def start(&callback)
      @req.start(&callback)
    end

    def url
      @url
    end

  end

  class Response
    attr_accessor :body, :headers, :status_code, :status_message
  end

end

module Licensing

  # NOTE: The language handler must initialized in the plugin elsewhere
  # This is just to prevent crashes where localization may not yet implemented in a particular
  # plugin that is using this license module
  LH = DMLanguageHandler.new("license.strings") unless defined?(LH)

  # NOTE: The constant 'PLUGIN' must be defined elsewhere and must
  # reference the parent module (which should always be the containing
  # module for this plugin. eg: DM::Artisan)

  # NOTE: The constant 'PLUGIN_NAME' must also be defined elsewhere and should reference
  # the standard display name for the plugin. (eg. 'Artisan' or 'Profile Builder 2')

  # NOTE: The constant 'PLUGIN::PATH' must be defined elsewhere and should reference
  # the folder that contains the main ruby files for this plugin

  # NOTE: The constant 'PLUGIN::VERSION' must be defined elsewhere and should reference
  # the current version number for this plugin (eg. 1.2.3)

  # NOTE: The constant 'PLUGIN::LIC_DIR' must be defined elsewhere and should reference
  # the directory name for the license file for this plugin

  # NOTE: The constant 'PLUGIN::LIC_FILE' must be defined elsewhere and should reference
  # the name for the license file for this plugin (eg. pb2.lic)

  ON_MAC = Sketchup.platform == :platform_osx
  SHARED_LOCATION = ON_MAC ? File.join(ENV['HOME'], 'Library', 'Application Support') : ENV['ALLUSERSPROFILE']
  MAC_LOCATION = ENV['systemroot'] ? File.join(ENV['systemroot'], 'System32') : '' #systemroot is a read-only variable so we will use it instead of windir
  require 'win32ole' unless ON_MAC

  def self.grant_full_access_windows(file_path) #grants everyone full access to file or folder
    if !ON_MAC #windows only
      params = File.exist?(MAC_LOCATION) ? "/c cd #{MAC_LOCATION} && " : '/c ' #change to normal icacls.exe location if folder exists
      params += "icacls \"#{file_path}\" /GRANT *S-1-1-0:(OI)(CI)(F)" #grant EVERYONE: folder, sub-folders, files, full access
      shell = WIN32OLE.new('Shell.Application')
      shell.ShellExecute('cmd.exe', params, '', 'open', 0)
    end
  end

  APP_DATA = File.join(SHARED_LOCATION, LIC_DIR)
  if !FileTest.exist?(APP_DATA) #file/folder does not exist
    Dir.mkdir(APP_DATA)  #create folder
    Licensing.grant_full_access_windows(APP_DATA) #grant all users full access
  end
  LOG_FILES = File.join(APP_DATA, 'Logs')

  DEBUG = 0   # Low-level information, mostly for developers.
  INFO = 1    # Generic (useful) information about system operation.
  WARN = 2    # A warning.
  ERROR = 3   # A handleable error condition.
  FATAL = 4   # An unhandleable error that results in a program crash.
  UNKNOWN = 5 # An unknown message that should always be logged.

  module Log
    # Logs will be saved into a sub folder in the license directory
    if File.exist?(APP_DATA)
      @logs_path = LOG_FILES
    else
      UI.messagebox(LH['Unable to find AppData directory.'])
      @logs_path = nil # Indicates error.
    end


    def self.create_log_file(logs_path)
      if logs_path.nil?
        raise(RuntimeError, 'No path to log files.') # This will be caught by the rescue at the bottom.
      end

      # Make sure the destination directory exist.
      unless File.directory?(logs_path)
        FileUtils.mkdir_p(logs_path) #create any of the folders that are missing
        Licensing.grant_full_access_windows(logs_path) #grant all users full access
      end

      if @log_file
        log_filename = @log_file.path #re-open the previously used log file
      else
        log_basename = PLUGIN.tmpname('log-') # Generates log files like: log-20141106-2652-1dpjrlb.log
        log_filename = File.join(logs_path, "#{log_basename}.log")
      end

      # Create the log file.
      mode = File::WRONLY | File::APPEND | File::CREAT #Open the file in write mode, append to existing file, create if needed
      log_file = File.open(log_filename, mode)

      log_file
    rescue
      # Account for permission failure and other errors. Fall back to logging
      # to a StringIO object. It won't be logged to file, but the extension can
      # still work.
      return StringIO.new
    end

    # Thread safe logger.
    # http://www.ruby-doc.org/stdlib-2.0.0/libdoc/logger/rdoc/Logger.html
    @logger = nil

    # Create an accessors that perform lazy initialization of the log file so
    # we only create a log file when the extension is ever being used.
    def self.logger
      @log_file = self.create_log_file(@logs_path) #creates or opens the log file
      @logger ||= Logger.new(@log_file) #set logger only if it is not already set
      @logger
    end

    def self.end_writing
      self.logger.close
      @logger = nil
    end

    # http://www.ruby-doc.org/stdlib-2.0.0/libdoc/logger/rdoc/Logger/Severity.html
    # DEBUG (0)   # Low-level information, mostly for developers.
    # INFO (1)    # Generic (useful) information about system operation.
    # WARN (2)    # A warning.
    # ERROR (3)   # A handleable error condition.
    # FATAL (4)   # An unhandleable error that results in a program crash.
    # UNKNOWN (5) # An unknown message that should always be logged.
    def self.write(string, type = DEBUG)
      array = [string.to_s] #make it a one element array so we can pack and unpack it
      encoded = array.pack('m')
      case(type)
      when DEBUG
        self.logger.debug{ encoded }
      when INFO
        self.logger.info{ encoded }
      when WARN
        self.logger.warn{ encoded }
      when ERROR
        self.logger.error{ encoded }
      when FATAL
        self.logger.fatal{ encoded }
      when UNKNOWN
      else
      end
      self.end_writing
    end

    def self.debug(string)
      self.write(string, type = DEBUG)
    end

    def self.error(string)
      self.write(string, type = ERROR)
    end

    def self.fatal(string)
      self.write(string, type = FATAL)
    end

    def self.info(string)
      self.write(string, type = INFO)
    end

    def self.warn(string)
      self.write(string, type = WARN)
    end

  end

  def self.open_log_file_location
    path = "file://#{LOG_FILES}"
    path.gsub!(" ", "%20") if (ON_MAC && Sketchup.version.to_i >= 19)
    UI.openURL(path)
  end

  # Action to be taken if an trial license is expired
  def self.on_trial_expired
    Sketchup.active_model.abort_operation # Should be safe to call this even if there is no current operation
    UI.messagebox(TRIAL_EXPIRED_MESSAGE)
    Sketchup.active_model.select_tool(nil)
    LicenseInfo.instance.show
  rescue
    # We want to make sure this operation will return since it may have been called from the c++
    # online license check block and we need to ensure that block completes execution
    return
  end

  # Action to be taken if an invalid license is found
  # @param [InvalidLicenseException] err sub-class of RuntimeError
  def self.on_invalid_license_found(err = nil)
    Sketchup.active_model.abort_operation # Should be safe to call this even if there is no current operation
    UI.messagebox(INVALID_LICENSE_MESSAGE)
    Sketchup.active_model.select_tool(nil)
    LicenseInfo.instance.show
  rescue
    # We want to make sure this operation will return since it may have been called from the c++
    # online license check block and we need to ensure that block completes execution
    return
  end

  # Action to be taken if subscription license is expired
  def self.on_license_expired
    Sketchup.active_model.abort_operation # Should be safe to call this even if there is no current operation
    UI.messagebox(LICENSE_EXPIRED_MESSAGE)
    Sketchup.active_model.select_tool(nil)
    LicenseInfo.instance.show
  rescue
    # We want to make sure this operation will return since it may have been called from the c++
    # online license check block and we need to ensure that block completes execution
    return
  end

  PLIST_LOCATION = File.join(SHARED_LOCATION, 'MSS')
  PLIST_FILE_NAME = 'active.plist'

  # Licensing LH messages
  SERVER_ERROR_MESSAGE =  LH["Unable to validate license. Please try again later or contact customer support for plugin: "] + PLUGIN_NAME
  ACTIVATIONS_EXCEEDED_MESSAGE = LH["You have exceeded the allowable number of activations for this license. Please remove a previously activated license from another computer and then try again."]
  VALIDATION_MESSAGE = LH["There was an issue validating your license. Please contact support and send your log files: "] + LOG_FILES #used for when validating a license fails
  EXCEEDED_ACTIVATION_ERROR = LH["You have exceeded the allowable number of activations for this license. Please contact customer support. Send a detailed description and your log files: "] + LOG_FILES
  LICENSE_FOUND_MESSAGE = LH["License found. Thank you for purchasing!"]
  TRIAL_FOUND_MESSAGE = LH["Trial License found."]
  SYSTEM_ID_INVALID = LH["System ID does not match expected format."]
  SYSTEM_ID_CHANGED = LH["System ID has changed."]
  ONLINE_LICENSE_CHECK_FAILED = LH["There was a problem validating your plugin license. Please re-activate your license for: "] + PLUGIN_NAME
  OFFLINE_REMOVE_LIC_MESSAGE = LH["You must be connected to the internet to remove your license."]
  OFFLINE_RENEW_LIC_MESSAGE = LH["You must be connected to the internet to renew your license."]
  TRIAL_EXPIRED_MESSAGE = LH["Your trial has expired for plugin: "] + PLUGIN_NAME
  INVALID_LICENSE_MESSAGE = LH["A valid license could not be found for plugin: "] + PLUGIN_NAME
  LICENSE_EXPIRED_MESSAGE = LH["Your license has expired. Please use the License Manager to renew your license for plugin: "] + PLUGIN_NAME
  TRIAL_ABUSE_MESSAGE = LH["Sorry, you have exceeded the allowable number of trial activations. Please purchase a license to keep using this product."]

  module HtmlDialogHelper
    def add_common_callbacks
      @dlg.add_action_callback("ready") {
        localize
        refresh_dialog
        disable_actions
      }
    end

    def show
      if @dlg && @dlg.visible?
        @dlg.bring_to_front
      else
        init_dialog
        add_action_callbacks # Window is being opened, NEED to add callback since HtmlDialogs lose connection when closed
        @dlg.show
        @dlg.center
      end
    end

    def add_slashes(text)
      val = ""
      if text
        val = text.dup
        val.gsub!(/\\/, "\\\\\\")
        val.gsub!(/\n/, "\\\\n")
        val.gsub!(/"/, "\\\"")
      end
      val
    end

    # Sets the value of an input field in the Html dialog
    def set_value(id, value, dlg = self)
      new_value = add_slashes(value.to_s)
      cmd = "document.getElementById('#{id}').value=\"#{new_value}\""
      dlg.execute_script(cmd)
    end

    def call_js(function, *arguments)
      js_args = arguments.map { |x| x.to_json }.join(", ")
      javascript = "#{function}(#{js_args});"
      @dlg.execute_script(javascript)
    end

    def localize
      call_js("localize", LH.strings)
    end

    def disable_actions
      call_js("disableContextMenu")
      call_js("disableSelect")
    end

  end # Htmldialog helper

  class LicenseInfo

    @@info = nil
    include Licensing::HtmlDialogHelper

    def self.instance
      @@info = new unless @@info
      return @@info
    end

    # PLUGIN set to plugin module (Artisan, PB2 ...)
    def initialize
      @dlg = nil
    end # init

    # Always initialize the dialog each time it is shown (needed to avoid dialog issues on Mac)
    def init_dialog
      dlg_name = LH["License Manager: "] + PLUGIN_NAME
      dlg_id = "#{PLUGIN_NAME}LicInfo"
      height = ON_MAC ? 320 : 330
      @dlg = UI::HtmlDialog.new(
          dialog_title: dlg_name,
          preferences_key: dlg_id,
          scrollable: false,
          resizable: true,
          width: 450,
          height: height,
          left: 150,
          top: 150
      )
      @dlg.set_file(File.join(PLUGIN::PATH, 'html', 'lic_info.html'))
    end

    # HtmlDialogs need to have their callbacks added any time they are opened as the connection is removed when the dialog is closed
    def add_action_callbacks
      add_common_callbacks
      @dlg.add_action_callback('ValueChanged') { |_, p|
        pair = p.split("|")
        id = pair[0]
        case id
        when "addLicense"
          if @lic.get_state == LICENSED
            UI.messagebox(LH["You already have a valid license."])
          else
            @lic.register {refresh_dialog}
          end
        when "renewLicense"
          # Note that the Renew License button should only be visible if the user has a paid or NFR license
          # and the license is in an expired state
          @lic.get_state
          if @lic.lic_for == "NFR"
            # This message should only appear for partners so we won't bother localizing it.
            UI.messagebox("Please contact customer support to request a new NFR license.")
          elsif @lic.lic_for == "Commercial"
            result = UI.messagebox(LH["Are you sure you want to renew and reactivate your license on this computer?"], MB_YESNO)
            if result == IDYES
              on_valid = ->  { refresh_dialog }
              on_invalid = -> (reason){ on_invalid_license_refresh(reason) }
              make_sure = -> {}
              @lic.renew_license(on_valid, on_invalid, make_sure)
            end
          else
            UI.messagebox(LH["Your license type can not be renewed. Please purchase a new license to continue using this product."])
          end
        when "removeLicense"
          result = UI.messagebox(LH["Are you sure you want to remove your license?"], MB_YESNO)
          if result == IDYES
            on_valid = -> { on_licensed_removed } #callback function
            on_invalid = -> (reason){ on_invalid_license_removal(reason) } #callback function
            make_sure = -> {} #callback function
            @lic.remove_license(on_valid, on_invalid, make_sure)
          end
        end
      }
    end

    def on_licensed_removed
      License.remove_license_file
      message = LH["Your license has been removed."]
      UI.messagebox(message)
      refresh_dialog
    end

    def on_invalid_license_refresh(reason)
      message = LH["There are no renewals available for this license. Please ensure your administrator has purchased "\
      "a license renewal for this account.  Would you like to purchase now?"]
      result = UI.messagebox(message, MB_YESNO)
      if result == IDYES
        UI.messagebox(LH["A website will open where you can purchase a license renewal for this product. Once you have received your renewed license information, please renew your license using the License Manager."])
        @lic.renew_subscription
      end
      Licensing::Log.warn(reason) # Log reason for invalid license
    end

    def on_invalid_license_removal(reason)
      License.remove_license_file
      Licensing::Log.warn(reason) # Log reason for invalid license
      refresh_dialog
    end

    def refresh_dialog
      @lic = License.new
      state = @lic.get_state
      product = @lic.product + " (#{PLUGIN::VERSION})"
      set_button_visibility(state)
      set_button_widths(state)
      set_value("product", product, @dlg)
      set_value("customer", @lic.customer, @dlg)
      set_value("email", @lic.email, @dlg)
      set_value("expires", @lic.expires, @dlg)
      set_value("type", get_friendly_license_type_name(@lic.lic_type), @dlg)
      set_value("for", @lic.lic_for, @dlg)
      set_value("licensor", @lic.licensor, @dlg)
      set_value("status", get_friendly_license_state_name(state), @dlg)
    end

    # sets the visibility of the buttons depending on the state
    def set_button_visibility(status)
      case status
      when NOT_LICENSED
        cmd = "$('#removeLicense').hide();$('#addLicense').show();$('#renewLicense').hide();"
      when TRIAL, TRIAL_EXPIRED, LICENSED, LICENSE_ERROR
        cmd = "$('#removeLicense').show();$('#addLicense').hide();$('#renewLicense').hide();"
      when LICENSE_EXPIRED
        cmd = "$('#removeLicense').show();$('#addLicense').hide();$('#renewLicense').show();"
      end
      @dlg.execute_script(cmd)
    end

    # sets the width of the buttons depending on the state
    def set_button_widths(status)
      case status
      when TRIAL, TRIAL_EXPIRED, LICENSED
        # one button displayed - remove
        cmd = "$('#removeLicense').width('90%');"
      when LICENSE_EXPIRED
        # two buttons displayed - remove or renew
        cmd = "$('#removeLicense').width('160px');$('#renewLicense').width('160px')"
      else
        # display a full width activate button
        cmd = "$('#addLicense').width('90%');"
      end
      @dlg.execute_script(cmd)
    end

    # Returns a friendly and / or localized type name for the license based on the data that was found in the database
    def get_friendly_license_type_name(type)
      if PLUGIN.respond_to?(:license_type)
        return PLUGIN.license_type(type)
      end
      name = ""
      case type
      when "SUS" # Subscription licenses
        name = LH["Single User Subscription"]
      when "single user", "SUP" # Permanent licenses
        name = LH["Single User"]
      when "Trial"
        name = LH["Trial"]
      else
        name = type
      end
      name
    end

    # Returns a friendly and / or localized name for the status of the license
    def get_friendly_license_state_name(status)
      name = ""
      case status
      when LICENSED
        if @lic.expires != "permanent"
          name = LH["Active - Days Remaining: "] + "#{@lic.days_remaining}"
        else
          name = LH["Licensed"]
        end
      when LICENSE_EXPIRED
        name = LH["License Expired"]
      when TRIAL
        if @lic.expires != "permanent"
          name = LH["Trial License - Days Remaining: "] + "#{@lic.days_remaining}"
        else
          name = LH["Licensed"]
        end
      when TRIAL_EXPIRED
        name = LH["Trial Expired"]
      when NOT_LICENSED
        name = LH["Not Licensed"]
      when LICENSE_ERROR
        name = LH["License Error"]
      end
      name
    end

  end #LicenseInfo

  class License

    include Licensing::HtmlDialogHelper

    FILE = File.join(Licensing::APP_DATA, LIC_FILE)
    COMPANY = "MindSight Studios Inc."

    def self.windows_mac_addr_string
      if !ON_MAC
        params = File.exist?(MAC_LOCATION) ? "#{MAC_LOCATION}/" : '' #change to normal ipconfig.exe location if folder exists
        params += "ipconfig /all" #add the ipconfig command that will execute (either we are in the correct folder or hopefully the system path is properly set)
      end
      return params
    end

    def self.file
      FILE
    end

    #gets host id that is stored in the plist to verify that the system is licensed
    def self.get_host_id_from_plist
      file_name = File.join(PLIST_LOCATION, PLIST_FILE_NAME)
      if !File.exist?(file_name)
        Log.error("Plist file: #{file_name} does not exist cannot get valid plist host_id")
      end

      if ON_MAC
        cmd = "defaults read \"#{file_name}\" \"#{PLUGIN_NAME}\"" #reads the value from the file
        host_id = %x[#{cmd}].rstrip #result will be the host id, nil if it didn't exist or failed
      else
        hash = self.read_plist_file
        host_id = hash.nil? ? nil : hash["#{PLUGIN_NAME}"]
      end

      return host_id ? host_id : '' #do not return nil
    end

    #opens the plist file to read if it exists, then gets the host id hash
    def self.read_plist_file
      file_name = File.join(PLIST_LOCATION, PLIST_FILE_NAME)
      hash = nil
      begin
        if File.exist?(file_name)
          out = File.new(file_name, 'rb')
          text = out.read
          out.close unless out.closed?
          hash = JSON.parse(text)
        end
      rescue
        out.close if out && !out.closed?
      end

      return hash
    end

    # opens and reads a file into an array for each line
    def self.read_file_to_array(file_path)
      f = File.new(file_path, "rb")
      lines = f.read.split("\n")
      lines.each { |line| line.strip! } # ensures \r character will also get removed (remnant from older license files)
      f.close
      lines
    end

    # return true if file can be written to
    def self.writable(file_path)
      return true unless Object.const_defined?("Pathname") # For SU 2016 Pathname is not valid so returning true
      foo = Pathname.new(file_path)
      foo.writable?
    end

    # creates the license file
    def self.write_file(file_path, data, serial, sig)
      out = File.new(file_path, "wb")
      out.puts "LICENSE\n"
      out.puts data + "\n"
      out.puts serial + "\n"
      out.puts sig + "\n"
      out.close unless out.closed?
    end

    #removes or overwrites the host id for the Plugin from the file
    def self.remove_host_id_from_plist
      file_name = File.join(PLIST_LOCATION, PLIST_FILE_NAME)
      if ON_MAC
        cmd = "defaults delete \"#{file_name}\" \"#{PLUGIN_NAME}\"" #deletes the key, value from the plist
        result = system(cmd) #result will be true if cmd was success and false otherwise
        #can check if it fails but that should mean the file or entry was already removed
      else
        begin
          if FileTest.exist?(file_name)
            hash = License.read_plist_file
            if hash.nil?
              File.delete(file_name)
            else
              hash.delete("#{PLUGIN_NAME}") #remove the key and therefore its value from the hash
              out = File.new(file_name, 'wb')
              out.puts(hash.to_json) #save new reduced plist json hash
              out.close unless out.closed?
            end
          end
        rescue Exception => e
          Log.error("Problem when removing: #{PLUGIN_NAME} entry from plist")
          Log.error(e.message)
          Log.info(e.backtrace)
            #todo UI message the user?
        ensure
          out.close if out && !out.closed?
        end
      end
    end

    def self.remove_license_file
      if FileTest.exist?(FILE)
        File.delete(FILE)
      end
    end

    def initialize
      @licensor = "" #needed so it can be displayed by LicenseInfo class
      @product = "#{PLUGIN_NAME}" #needed so it can be displayed by LicenseInfo class
      @customer = LH["Unregistered User"] #needed so it can be displayed by LicenseInfo class
      @email = "" #needed so it can be displayed by LicenseInfo class
      @type = "" #single user, network, trial #needed so it can be displayed by LicenseInfo class
      @lic_for = "" #trial, commercial, educational, nfr #needed so it can be displayed by LicenseInfo class
      @expires = "" #needed so it can be displayed by LicenseInfo class
      @days_remaining = 0 #needed so it can be displayed by LicenseInfo class
      @host_id = "" #the selected host_id #needed so it can be written to plist
      @callback = nil
      @dlg = nil
    end

    # Always initialize the dialog each time it is shown (needed to avoid dialog issues on Mac)
    def init_dialog
      dlg_name = LH["Activate License: "] + PLUGIN_NAME
      dlg_id = "Reg#{PLUGIN_NAME}"
      height = ON_MAC ? 165 : 175
      @dlg = UI::HtmlDialog.new(
          dialog_title: dlg_name,
          preferences_key: dlg_id,
          scrollable: false,
          resizable: true,
          width: 500,
          height: height,
          left: 150,
          top: 150
      )
      rb_initialize # Sets the activation url
    end

    # HtmlDialogs need to have their callbacks added any time they are opened as the connection is removed when the dialog is closed
    def add_action_callbacks
      return if @dlg.visible?

      @dlg.add_action_callback("create_license") { |_, data|
        cb_create_license(data)
      }
      @dlg.add_action_callback("lic_not_found") {
        @dlg.close
        if ON_MAC
          text = LH["Sorry, your license key does not appear to be valid.  Please try again and be sure to use copy / paste (CMD+C and CMD+V) to enter the key from your license information email."]
        else
          text = LH["Sorry, your license key does not appear to be valid.  Please try again and be sure to use copy / paste (CTRL+C and CTRL+V) to enter the key from your license information email."]
        end
        UI.messagebox(text)
      }
      @dlg.add_action_callback("entitlement_exceeded") {
        @dlg.close
        UI.messagebox(ACTIVATIONS_EXCEEDED_MESSAGE)
        PLUGIN.on_activations_exceeded if PLUGIN.respond_to?(:on_activations_exceeded)
      }
      @dlg.add_action_callback("set_the_product") {
        set_product
        set_host_ids
      }
      @dlg.add_action_callback("open_url") do |_, p|
        p.gsub!(" ", "%20") if (ON_MAC && Sketchup.version.to_i >= 19)
        UI.openURL(p)
      end
    end

    # Over-ride inherited Object methods to help protect class
    def extend(*args)
    end
    def inspect
      ""
    end
    def instance_eval(*args)
    end
    def instance_variables
      []
    end
    def method(*args)
    end
    def methods
      []
    end
    def private_methods
      []
    end
    def public_methods
      []
    end
    def protected_methods
      []
    end
    def respond_to?(*args)
    end
    def send(*args)
    end
    def singleton_methods
      []
    end
    def instance_methods
      []
    end
    def public_instance_methods
      []
    end
    def protected_instance_methods
      []
    end
    def private_instance_methods
      []
    end
    def to_a
      []
    end
    def to_s
      ""
    end
    def constants
      []
    end
    def class_variables
      []
    end

    public
    def register(&block)
      is_online = -> {
        @callback = block
        rb_register
      }
      is_offline = -> {
        Log.warn("Sketchup.is_online returned FALSE")
        UI.messagebox(LH["No internet connection can be found.  You must be "\
                          "connected to the internet to activate your license."])
      }
      Http.ping(is_online, is_offline)
    end

    def host_id
      @host_id.dup
    end

    def product # needed so it can be displayed by LicenseInfo class
      if PLUGIN.respond_to?(:product_name)
        # Allows the plugin to display a different product name depending on the
        # license info
        PLUGIN.product_name(self)
      else
        @product.dup
      end
    end

    def customer #needed so it can be displayed by LicenseInfo class
      @customer.dup
    end

    def email #needed so it can be displayed by LicenseInfo class
      @email.dup
    end

    def expires #needed so it can be displayed by LicenseInfo class
      @expires.dup
    end

    def serial #needed for certain functions
      @serial.dup
    end

    def lic_type #needed so it can be displayed by LicenseInfo class
      @type.dup
    end

    def lic_for #needed so it can be displayed by LicenseInfo class
      if PLUGIN.respond_to?(:lic_for)
        # Allows the plugin to over-write lic for status
        PLUGIN.lic_for(@lic_for)
      else
        @lic_for.dup
      end
    end

    def licensor #needed so it can be displayed by LicenseInfo class
      @licensor.dup
    end

    def days_remaining #needed so it can be displayed by LicenseInfo class
      @days_remaining
    end

    private
    def set_product
      set_value('productSU', PLUGIN_NAME, @dlg)
    end

    #creates the file if needed then adds the host id to it with the Plugin name as the key
    def add_host_id_to_plist
      file_name = File.join(PLIST_LOCATION, PLIST_FILE_NAME)
      if ON_MAC
        cmd = "defaults write \"#{file_name}\" \"#{PLUGIN_NAME}\" -string \"#{@host_id}\"" #creates folders and file if needed then sets key value
        result = system(cmd) #result will be true if cmd was success and false otherwise
        if result == false
          raise("Failed to add host id: #{@host_id} to the plist")
        end
      else
        Dir.mkdir(PLIST_LOCATION) unless FileTest.exist?(PLIST_LOCATION)
        Licensing.grant_full_access_windows(PLIST_LOCATION)#change permission full access to everyone
        begin
          hash = License.read_plist_file
          out = File.new(file_name, 'wb')
          hash.nil? ? hash = { "#{PLUGIN_NAME}" => @host_id } : hash["#{PLUGIN_NAME}"] = @host_id
          out.puts(hash.to_json) #save new complete plist json hash
          out.close unless out.closed?
        ensure
          out.close if out && !out.closed?
        end
      end
    end

    # This class is frozen in c++
  end

  # This method must be defined in the plugin loader.rb file prior to this file being loaded
  PLUGIN.load_c_extensions

  freeze # module Licensing

end

class AutoUpdate
  FILENAME = File.join(File.dirname(Licensing::License::FILE), "#{PLUGIN_NAME}.AutoUpdate.rbz").freeze #using lic_dir since we know this should be usable to all users
  SECTION = "DM #{PLUGIN_NAME}".freeze
  VARIABLE = 'AutoUpdate'.freeze
  AUTO_CHECK_ID = 'AutoCheck'.freeze
  @@settings = nil
  @@update_available = false
  @@checked = false
  REMOTE_PATH = "https://0.0.0.0/".freeze
  REMOTE_DIST = "https://0.0.0.0/".freeze
  include Licensing::HtmlDialogHelper

  GREATER = 100
  EQUAL = 150
  LESS = 300

  def self.instance(&block)
    if @@settings
      @@settings.set_callback{ block.call if block }
    else
      @@settings = new { block.call if block }
    end
    return @@settings
  end

  def self.auto_check_enabled
    Sketchup.read_default(SECTION, VARIABLE, 'true').to_s == 'true' ? true : false #defaults to enabled (true) also ensures we get a true or false only
  end

  def self.check_for_update(auto_check = false, &block)
    block.call if block
    return nil

    @@update_available = false

      link = "#{REMOTE_DIST}update-check.php?name=#{PLUGIN_NAME}"
      link.gsub!(/\s/, "%20")
      @req = Http::Request.new(link)
      @req.start do |req, res|
        if res.status_code == 200
          begin
            result = JSON.parse(res.body)
            version = Sketchup.version.to_i
            if version >= result['min']
              if version <= result['max']
                if self.is_version_newer?(result['version'])
                  # Newer version of this plugin is available so show the update dialog
                  @@update_available = true
                  self.instance{ block.call if block }.show
                next
                end
              end
            else
            UI.messagebox(LH["The update requires a newer version of SketchUp."] + " (#{result['min']})") unless auto_check # SU version needs to be updated to update plugin
              #todo raise special error
            end
          rescue Exception => e
            # todo json parse error, Should not get here
          end
      elsif res.status_code == 0
      # User is not online
      if !auto_check
        # If this was a manual check, tell the user that they must be online
        UI.messagebox(LH["Please ensure you are online before checking for an update."])
          next
      else # user did not manually check for update. Pretend it didn't happen
        block.call if block
          next
      end
      else
        # Response status code was not HTTP 200 success
        UI.messagebox(LH["HTTP request to check for update failed."]) #the http call to check available rbz failed
    end

    # If we got here, the user was online but there was no update available
    if auto_check
      # Execute the callback block (typically launching a tool or dialog)
      block.call if block
    else
      # If this was manual check, display the update dialog to notify the user that no update is available
      self.instance{ block.call if block }.show
    end
    end

  end

  #compare versions and see if this is an update
  def self.is_version_newer?(new_version)
    version_regex = /^(?<major>\d+)\.(?<minor>\d+)\.(?<other>\d+)$/
    current_v = VERSION
    if !version_regex.match(current_v)
      #puts "Your current version number is invalid"
      return false
    end
    if !version_regex.match(new_version)
      #puts "The remote version number is invalid"
      return false
    end

    major = self.compare_part(new_version, current_v, version_regex, 'major')
    case major
    when GREATER
      return true;
    when LESS
      return false;
    end

    minor = self.compare_part(new_version, current_v, version_regex, 'minor')
    case minor
    when GREATER
      return true;
    when LESS
      return false;
    end

    other = self.compare_part(new_version, current_v, version_regex, 'other')
    return true if other == GREATER

    return false
  end

  # use regex to compare different parts of 2 strings
  def self.compare_part(new, cur, regex, part)
    update = new.match(regex)[part].to_i
    current = cur.match(regex)[part].to_i
    if update > current
      return GREATER
    elsif update == current
      return EQUAL
    else
      return LESS
    end
  end

  #PLUGIN set to plugin module (Artisan, PB2 ...)
  def initialize(&block)
    @callback = block
    @dlg = nil
    @auto_check = AutoUpdate.auto_check_enabled
  end #init

  def init_dialog
    dlg_name = LH["Update: "] + PLUGIN_NAME
    dlg_id = "#{PLUGIN_NAME}AutoUpdate"
    @dlg = UI::HtmlDialog.new(
        dialog_title: dlg_name,
        preferences_key: dlg_id,
        scrollable: false,
        resizable: true,
        width: 450,
        height: 200,
        left: 150,
        top: 150
    )
    url = "#{REMOTE_PATH}update-dialog.php?plugin=#{PLUGIN_NAME}&update=#{@@update_available}"
    url.gsub!(/\s/, "%20")
    @dlg.set_url(url)
  end

  # HtmlDialogs need to have their callbacks added any time they are opened as the connection is removed when the dialog is closed
  def add_action_callbacks
    @dlg.add_action_callback('initialize') { |_, p|
      js_cmd = "document.getElementById('#{AUTO_CHECK_ID}').checked = #{@auto_check}; updateValue();"
      @dlg.execute_script(js_cmd)
      localize
      disable_actions
    }
    @dlg.add_action_callback('installUpdate') { |_, p|
      download_rbz
    }
    @dlg.add_action_callback('openChangelog') { |_, p|
      url = "#{REMOTE_DIST}update-check.php?name=#{PLUGIN_NAME}&changelog=1"
      url.gsub!(" ", "%20") if (ON_MAC && Sketchup.version.to_i >= 19)
      UI.openURL(url)
    }
    @dlg.add_action_callback('auto_check') do |_, value|
      @auto_check = value
    end
    @dlg.set_on_closed {
      Sketchup.write_default(SECTION, VARIABLE, @auto_check)
      @callback.call if @callback
      @callback = nil
    }
  end

  def set_callback(&block)
    @callback = block
  end

  def download_rbz
    filename = FILENAME
    @callback = nil # clear callback since user should restart
    url = "#{REMOTE_DIST}download.php?name=#{PLUGIN_NAME}"
    url.gsub!(/\s/, "%20")
    @rbz_req = Http::Request.new(url)
    @rbz_req.set_download_progress_callback do |current, total|
      percentage = (current.to_f / total * 100).round
      cmd = "setDownloadProgress(#{percentage})"
      @dlg.execute_script(cmd)
    end
    @dlg.set_on_closed do
      Licensing::Log.warn("RBZ Download was cancelled") # Log
      @rbz_req.cancel
    end

    @rbz_req.start do |req, resp|
      case resp.status_code
      when 200
        file_data = resp.body
        File.open(filename, "wb:BINARY") { |temp_file|
          temp_file.write(file_data)
        }
        result = extract_rbz
        UI.messagebox(LH["The update is not finished yet. Please close all SketchUp windows and restart to load the update."]) if result
        @dlg.close
      else
        UI.messagebox(LH["Error downloading the update. Please ensure you are online."])
      end
    end
  end

  def extract_rbz
    path = FILENAME
    begin
      Sketchup.install_from_archive(path)
      return true
    rescue Interrupt => error
      UI.messagebox(LH["Update was canceled."])
    rescue Exception => error
      UI.messagebox(LH["An error occured while installing the update. Please manually download the update from the website."])
    end
    return false
  end

end

end # plugin module
# This file is generated from the build_license.rb script. 
# Do not edit this file manually.

module DM::Bevel

  def self.get_icon(name)
    ext = ON_MAC ? ".pdf" : ".svg"
    name += ext unless name.include?(".") # add SVG extension unless an extension was directly provided
    File.join(PATH, "icons/#{name}")
  end

  def self.get_command(name, icon, tooltip, status_text, check_license, tool_class = nil, &cmd_block)
    cmd = UI::Command.new(name) {
      cmd_block = Proc.new { Sketchup.active_model.select_tool(tool_class.new) } if tool_class
      if check_license
        AutoUpdate.check_for_update(true) { cmd_block.call if PLUGIN::check_license }
      else
        cmd_block.call
      end
    }
    cmd.large_icon = cmd.small_icon = get_icon(icon)
    cmd.tooltip = tooltip
    cmd.status_bar_text = status_text
    if tool_class
      cmd.set_validation_proc do
        if PLUGIN::CoreTool.active_tool.class == tool_class
          MF_CHECKED
        else
          MF_ENABLED
        end
      end
    end
    cmd
  end

  def self::build_ui
    main_menu = UI.menu("Plugins")
    toolbar = UI::Toolbar.new(PLUGIN_NAME)
    main_menu.add_separator
    main_menu = main_menu.add_submenu(PLUGIN_NAME)

    cmd = get_command(LH["Bevel Tool"],
                "bevel_tb",
                LH["Bevel Tool"],
                LH["Create chamfered or rounded edges"],
                true,
                BevelTool)
    toolbar.add_item(cmd)
    main_menu.add_item(cmd)

    cmd = get_command(LH["Live Bevel"],
                      "bevel-nd_tb",
                      LH["Live Bevel"],
                      LH["Add a Non-Destructive Bevel to the selected group or component"],
                      true) { BevelNonDestructive.non_destructive }
    toolbar.add_item(cmd)
    main_menu.add_item(cmd)

    cmd = get_command(LH["Auto Soften Edges"],
                      "bevel-soft_tb",
                      LH["Auto-soften and smooth edges"],
                      LH["Improve bevel results by softening edges before or after beveling"],
                      true) { PLUGIN.soften_edges(Sketchup.active_model.selection.to_a) }
    toolbar.add_item(cmd)
    main_menu.add_item(cmd)

    cmd = get_command(LH["Clean Edges"],
                      "bevel-clean_tb",
                      LH["Clean Edges"],
                      LH["Improve bevel results by removing coplanar and split edges prior to beveling"],
                      true) { PLUGIN.clean_coplanar_collinear_edges }
    toolbar.add_item(cmd)
    main_menu.add_item(cmd)

    cmd = get_command(LH["Preferences"],
                      "bevel-settings_tb",
                      LH["Preferences"],
                      LH["Edit settings and preferences"],
                      true) { BevelSettingsDialog.instance.show }
    toolbar.add_item(cmd)
    main_menu.add_item(cmd)

    cmd = get_command(LH["Help"],
                      "",
                      LH["Help"],
                      LH["Get support from the mind.sight.studios Help Desk"],
                      false) { UI.openURL("https://support.mindsightstudios.com") }
    main_menu.add_item(cmd)

    # This is needed to properly restore the previous toolbar state
    state = toolbar.get_last_state
    case state
    when 1, -1
      toolbar.show
    when 0
      toolbar.hide
    end

    main_menu.add_separator
    cmd = UI::Command.new(LH["License.."]){ AutoUpdate.check_for_update(true) { Licensing::LicenseInfo.instance.show } }
    cmd.tooltip = cmd.status_bar_text = LH["Show licensing information"]
    main_menu.add_item(cmd)

    cmd = UI::Command.new(LH["Check for Update.."]) { AutoUpdate.check_for_update(false) }
    cmd.tooltip = cmd.status_bar_text = LH["Open Update Dialog"]
    main_menu.add_item(cmd)

  end

  self::build_ui


  def self.parse_length(string)
    string.strip! # Trialing space will fail
    string.gsub!(/”|“/, '"') # Curly quotes will fail
    string.gsub!(/‘|’/, "'")
    string.gsub!(/-\s+/,'-')
    if string.match("'")
      string += '"' unless string[-1] == '"' # Since we found ' assume user is dealing with feet and inches
    end
    return nil if Sketchup.parse_length(string) == nil
    Sketchup.parse_length(string) >= 0.0 ? Sketchup.parse_length(string) : -(Sketchup.parse_length(string.gsub('-', '')))
  end

  # @param message [String]
  # @param icon [String] Icon file name. (Don't include path)
  def self.notification(message, icon_filename = nil)
    return unless Sketchup.read_default(MAIN_DICTIONARY, BevelSettingsDialog::ID_SHOW_NOTIFICATION, true)
    
    if icon_filename.nil?
      icon = nil
    else
      icon = File.join(File.dirname(__FILE__), "icons", icon_filename)
    end

    @notification = UI::Notification.new(@@ext, message, icon, nil)
    @notification.show
  end

end

